function run_hybrid_worker_for_phase_boundary(batchParams, batchIdx)
    % Set up local cluster for parallel pool
    cluster                    = parcluster('local');
    nprocs                     = str2double(getenv('SLURM_CPUS_PER_TASK'));
    if isnan(nprocs), nprocs   = feature('numcores'); end

    tmpdir                     = fullfile(getenv('TMPDIR'), sprintf('matlab_job_%d', batchIdx));
    if ~exist(tmpdir, 'dir'); mkdir(tmpdir); end
    cluster.JobStorageLocation = tmpdir;

    pool = parpool(cluster, nprocs);

    nJobs = size(batchParams, 1);
    parfor k = 1:nJobs
        % Unpack parameter tuple
        initial_a_s  = batchParams(k, 1);
        theta_deg    = batchParams(k, 2);
        phi_deg      = batchParams(k, 3);
        N_atoms      = batchParams(k, 4);
    
        theta_rad    = deg2rad(theta_deg);
        phi_rad      = deg2rad(phi_deg);
    
        adjusted_a_s = initial_a_s;
    
        % Create unique save directory
        jobName = sprintf('aS_%03d_theta_%03d_phi_%03d_N_%d', initial_a_s, theta_deg, phi_deg, N_atoms);
        saveDir = fullfile('./Results/Data_3D/GradientDescent', jobName);
        if ~exist(saveDir, 'dir')
            mkdir(saveDir);
        end
        
        srcFile = './Results/Data_3D/GradientDescent/psi_init.mat';
        if exist(srcFile, 'file')
            copyfile(srcFile, fullfile(saveDir, 'psi_init.mat'));
        end
    
        MaxAttempts  = 10;
        SuccessFlag  = false;
        AttemptCount = 0;
    
        while ~SuccessFlag && AttemptCount < MaxAttempts
            AttemptCount = AttemptCount + 1;
    
            % Options for this run
            OptionsStruct = struct;
            OptionsStruct.NumberOfAtoms           = N_atoms;
            OptionsStruct.DipolarPolarAngle       = theta_rad;
            OptionsStruct.DipolarAzimuthAngle     = phi_rad;
            OptionsStruct.ScatteringLength        = adjusted_a_s;
    
            OptionsStruct.TrapFrequencies         = [50, 20, 150];
            OptionsStruct.TrapPotentialType       = 'Harmonic';
    
            OptionsStruct.NumberOfGridPoints      = [128, 256, 128];
            OptionsStruct.Dimensions              = [40, 80, 40];
            OptionsStruct.UseApproximationForLHY  = true;
            OptionsStruct.IncludeDDICutOff        = true;
            OptionsStruct.CutoffType              = 'Cylindrical';
            OptionsStruct.SimulationMode          = 'ImaginaryTimeEvolution'; % 'ImaginaryTimeEvolution' | 'RealTimeEvolution' | 'EnergyMinimization'
            OptionsStruct.GradientDescentMethod   = 'NonLinearCGD';           % 'HeavyBall' | 'NonLinearCGD' 
            OptionsStruct.MaxIterationsForGD      = 15000;
            OptionsStruct.TimeStepSize            = 1E-3;            % in s
            OptionsStruct.MinimumTimeStepSize     = 1E-6;            % in s
            OptionsStruct.TimeCutOff              = 1E6;             % in s
            OptionsStruct.EnergyTolerance         = 5E-10;
            OptionsStruct.ResidualTolerance       = 1E-05;
            OptionsStruct.NoiseScaleFactor        = 0.010;
    
            OptionsStruct.PlotLive                = false;
            OptionsStruct.JobNumber               = 0;
            OptionsStruct.RunOnGPU                = true;
            OptionsStruct.SaveData                = true;
            OptionsStruct.SaveDirectory           = saveDir;
    
            options = Helper.convertstruct2cell(OptionsStruct);
    
            sim = Simulator.DipolarGas(options{:});
            pot = Simulator.Potentials(options{:});
            sim.Potential = pot.trap();
    
            NumberOfOutputs = 5;
            try
                [Params, Transf, psi, ~, ~, stats] = Helper.runWithProfiling(@() sim.run(), NumberOfOutputs, saveDir);
    
                if Scripts.determineDensityModulation(psi, Params, Transf)
                    SuccessFlag = true;
                    runDir  = fullfile(saveDir, sprintf('Run_%03d', OptionsStruct.JobNumber));
                    psiFile = fullfile(runDir, 'psi_gs.mat');
    
                    if exist(psiFile, 'file')
                        Scripts.saveUpdatedScatteringLength(psiFile, adjusted_a_s, AttemptCount);
                    else
                        warning('Expected file %s not found. Cannot save final a_s.', psiFile);
                    end
                else
                    adjusted_a_s = adjusted_a_s - 0.2;  % Tweak as per your needs
                end
    
            catch ME
                fprintf('ERROR in job %d (attempt %d):\n%s\n', k, AttemptCount, getReport(ME, 'extended'));
                adjusted_a_s = adjusted_a_s - 0.2;
            end
        end
    
        if SuccessFlag
            fprintf('Batch %d | Job %d: a_s = %.2f, theta = %d°, phi = %d°, N = %d | Time = %.2f s\n', ...
                batchIdx, k, adjusted_a_s, theta_deg, phi_deg, N_atoms, stats.runtime);
        else
            fprintf('Batch %d | Job %d FAILED after %d tries.\n', batchIdx, k, MaxAttempts);
        end
    end

    delete(pool);
end